package cn.iododer.yudao.module.mqtt.api.path;

import cn.iocoder.yudao.module.mqtt.api.path.PathPlanningApi;
import cn.iocoder.yudao.module.mqtt.api.path.dto.PositionMapItemSynDTO;
import cn.iocoder.yudao.module.mqtt.api.path.dto.PositionMapLinePathDTO;
import cn.iododer.yudao.module.mqtt.util.MqttUtils;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController // 提供 RESTful API 接口，给 Feign 调用
@Validated
@Slf4j
public class PathPlanningApiImpl implements PathPlanningApi {

    @Autowired
    private MqttUtils mqttUtils;

    /**
     * 同步ware_position_map_line的点位信息
     * @param relatedPathNode
     * @param topic
     */
    @Override
    public void synchronousPointToPP(PositionMapLinePathDTO relatedPathNode, String topic) {
        try {
            mqttUtils.pub(topic, JSON.toJSONString(relatedPathNode));
            log.info("同步给路径规划完成 :{}", JSON.toJSONString(relatedPathNode));
        } catch (MqttException e) {
            log.info("同步点位信息给路径规划异常 :{}",e);
        }
    }

    /**
     * 同步ware_position_map_item的点位信息
     * @param positionMapItemSynDTOS
     * @param topic
     */
    @Override
    public void synchronousAllItemToPP(List<PositionMapItemSynDTO> positionMapItemSynDTOS, String topic) {
        try {
            mqttUtils.pub(topic, JSON.toJSONString(positionMapItemSynDTOS));
            log.info("同步ware_position_map_item的点位信息--完成 :{}", JSON.toJSONString(positionMapItemSynDTOS));
        } catch (MqttException e) {
            log.info("同步ware_position_map_item的点位信息--异常 :{}",e);
        }
    }

    @Override
    public void synchronousLineObject(Object obj, String topic) {
        log.info("收到请求 :{}", JSON.toJSONString(obj));
        try {
            mqttUtils.pub(topic, JSON.toJSONString(obj));
            log.info("同步信息给PP--完成 :{}", JSON.toJSONString(obj));
        } catch (MqttException e) {
            log.info("同步信息给PP--完成--异常 :{}",e);
        }
    }
}
